package GetStatusListing;            # SPECIALISATION WITHOUT BATCH SUB-SYSTEM

require Exporter;
@ISA = qw(Exporter);

@EXPORT_OK = qw(get_status_listing);

use Reporting qw(start_report failed_report ok_report debug_report command_report);

use strict;

# Queries the BSS for the state of (at least) all Unicore jobs
# and returns a list of their identifier together with the
# stats.
#
# Called in response to a #TSI_JOBQUERY in the script

# uses functions from Reporting

# No args
#
# Returns void to TSI
#         fail/success to NJS and a list of jobs
#
#        First line is "QSTAT",
#        followed by a line per found job, first word is BSS job identifier
#        and the second word is one of RUNNING, QUEUED or SUSPENDED
#
#        List must include all Unicore jobs (but can contain  extra jobs)
#
sub get_status_listing {

     my($command, $output, $result, $bssid, @lines, $junk, $bssid, $state, $data, $ustate, $jobname);

    start_report("Finding all jobs from Unicore");

    # Form the request command. 
    $command = "$main::qstat_cmd";

    command_report($command);

    $output = `($command) 2>&1`;

    # Parse output
    if($? != 0) {
         failed_report($output);
    }
    else {
        # Command succeeded. Parse the output and return
        # a line for each job found with two words, the first
        # is the job id the second its _Unicore_ state

        # There can be many TSIs running, return about all Jobs

        $result = "QSTAT\n";

        @lines = split /\n/,$output;

        for(@lines) {

            SWITCH: {

                /UNICORE_\d+_/ && do {
                    # Found a job running for Unicore
                    ($state,$junk) = split / /,$_;

                    # Perl passes command line through "sh -c" to
                    # interpret the commands which results in two processes
                    # matching. Use the one that is executing the command

                    # Except that now (17-6-99) both processes start with "sh -c"
                    # so pass back two states (the same I hope)

                    #if( ! ($_ =~ /sh -c/) ) {

                        ($junk,$bssid,$jobname) = split /_/,$_;

                        $ustate = "UNKNOWN";
                        $ustate = "RUNNING" if $state =~ /O|R|S/ ;
                        $ustate = "SUSPENDED"    if $state =~ /T/ ;

                        # Add to the returned lines, only if
                        # it fitted the Unicore format (has bssid)
			if ($jobname =~ /BBaattcchhSSiimmuullaattiioonn/) {
			    $bssid = "";
			}
                        if($bssid ne "") {
                            $result = "$result $bssid $ustate\n";
                        }
                    #}

                    last SWITCH;
                }

            }
        }

        print main::CMD_SOCK $result;
        debug_report("qstat executed OK\n$result");
    }
}
#
# $Id: GetStatusListing.pm,v 1.17 2002/04/09 17:42:18 svdb Exp $
#
# 04-05-01 Sven van den Berghe fecit Created from 3.0 TSI for Unicore 3.5
#
#                  Copyright (c) Fujitsu Ltd 2000 - 2002
#
#     Use and distribution is subject to the Community Source Licence.
#                                                       
# Further information is available from www.unicore.org or arcon@fecit.co.uk.
#
