package JobControl;

require Exporter;
@ISA = qw(Exporter);

@EXPORT_OK = qw(abort_job cancel_job hold_job resume_job);

use Reporting qw(start_report failed_report ok_report command_report);

use NoBatchOnly qw(get_target_pid);

use strict;

# Emulation of BSS job control functions

# Abort a job on the BSS.
#
# arg1: text from NJS with TSI_BSSID <request id>
#
sub abort_job {

    my $from_njs = shift;

    my ($requestid, $output, $command, $target_pid);

    ($requestid) = ($from_njs =~ /.*#TSI_BSSID (.+)\n/);

    start_report("Aborting request $requestid");

    $target_pid = get_target_pid($requestid);

    if($target_pid == -1) {
        failed_report("Target job not found");
		return;
    }

    # Ensure that $target_pid is a number
    $target_pid =~ /(\d+)/;
    $command = "kill -9 $1";

    command_report($command);

    $output = `($command) 2>&1`;

    # Parse output
    if($? != 0) {
        failed_report($output);
    }
    else {
        ok_report($output);
    }
}



# Cancel a job on the BSS (abort plus dispose of output files)
#
# arg1: text from NJS with TSI_BSSID <request id>
#
sub cancel_job {

    my $from_njs = shift;

    my ($requestid, $output, $command, $target_pid);

    ($requestid) = ($from_njs =~ /.*#TSI_BSSID (.+)\n/);

    start_report("Cancelling request $requestid");

    $target_pid = get_target_pid($requestid);

    if($target_pid == -1) {
        failed_report("Target job not found");
		return;
    }

    $command = "kill -9 $target_pid\n";

    command_report($command);

    $output = `($command) 2>&1`;

    # Parse output
    if($? != 0) {
        failed_report($output);
    }
    else {

		# NO: fetching the results in EndProcessing does this?
        # and delete the implicit output
#		my $out_name;
#		my $err_name;

        # Get the current directory (selecting files ending in the request id
        # preceded by a character preceded by a "." (e.g. xxxxx.[oe]requestod))
        # Twice is excessive though, try to do better sometime
#        opendir(DIR,".");
#        ($out_name) = grep { /\.o$requestid/ } readdir(DIR);
#        closedir(DIR);
#        opendir(DIR,".");
#        ($err_name) = grep { /\.e$requestid/ } readdir(DIR);
#        closedir(DIR);
#
#        unlink $out_name;
#        unlink $err_name;

        # Succeeded,
        ok_report($output);
    }
}


# Hold a job on the BSS.
#
# arg1: text from NJS with TSI_BSSID <request id>
#
sub hold_job {
    ok_report("No holding of jobs on TSI without Batch Sub-System")
}

# Resume a job on the BSS.
#
# arg1: text from NJS with TSI_BSSID <request id>
#
sub resume_job {
    ok_report("No holding of jobs on without Batch Sub-System")
}
# $Id: JobControl.pm,v 1.17 2002/04/09 17:42:18 svdb Exp $
#
# 04-05-01 Sven van den Berghe fecit Created from 3.0 TSI for Unicore 3.5
#
#                  Copyright (c) Fujitsu Ltd 2000 - 2002
#
#     Use and distribution is subject to the Community Source Licence.
#                                                       
# Further information is available from www.unicore.org or arcon@fecit.co.uk.
#
