package NoBatchOnly;

require Exporter;
@ISA = qw(Exporter);

@EXPORT_OK = qw(get_target_pid);

use Reporting qw(failed_report command_report);

use strict;

#
# Get the actual process id for a Unicore "job" being
# executed by this TSI
#
# arg1: the Unicore identifier of the job (-1 on failure -
#       which includes job not found)
#
# return: the process id
#
sub get_target_pid {

    my $unicoreid = shift;

    my $target_pid = -1;

    # Cannot know child id when starting execution of the Unicore job
    # so the NJS returns a generated value
    # we now need to convert this value into a real pid. At the moment by doing
    # a ps and processing (sometime by writing to a file in the child?)

    my $command = "$main::pspid_cmd";

    command_report($command);

    my $output = `($command) 2>&1`;

    # Parse output
    if($? != 0) {
         failed_report($output);
        return;
    }
    else {

        my @lines = split /\n/,$output;
		my $pid;
		my $bssid;
		my $junk;

        for(@lines) {

            SWITCH: {

                /UNICORE/ && do {
                    # Found a job running for Unicore
                    s/^\s+//; # remove leading spaces for short pids
                    ($pid,$junk) = split / /,$_;

                    # Perl passes command line through "sh -c" to
                    # interpret the commands which results in two processes
                    # matching. Use the one that is executing the command

                    if( ! ($_ =~ /sh -c/) ) {

                        ($junk,$bssid,$junk) = split /_/,$_;

                        # Is it a Unicore job?
                        if(!($bssid eq "")) {
                            # Is it the target?
                            if($bssid == $unicoreid) {
				# untaint the data
				$pid =~ /(\d+)/;
                                $target_pid = $1;
                            }
                        }
                    }

                    last SWITCH;
                }

            }
        }

    }

    return $target_pid;
}
#
# $Id: NoBatchOnly.pm,v 1.17 2002/04/09 17:42:18 svdb Exp $
#
# 04-05-01 Sven van den Berghe fecit Created from 3.0 TSI for Unicore 3.5
#
#                  Copyright (c) Fujitsu Ltd 2000 - 2002
#
#     Use and distribution is subject to the Community Source Licence.
#                                                       
# Further information is available from www.unicore.org or arcon@fecit.co.uk.
#
