package MainLoop;

require Exporter;
@ISA = qw(Exporter);

@EXPORT_OK = qw(infinite_loop);

use BecomeUser       qw(become_user restore_id);
use Reporting        qw(failed_report report_and_die);
use GetStatusListing qw(get_status_listing);
use Submit           qw(submit);
use ExecuteScript    qw(execute_script);
use EndProcessing    qw(end_processing);
use PutFiles         qw(put_files);
use GetDirectory     qw(get_directory);
use JobControl       qw(abort_job cancel_job hold_job resume_job);
use Dump2File        qw(dump2file);

use strict;

# Loop listening for input from NJS and processing it

# Writes directly to main::CMD_SOCK, reads directly from main::CMD_SOCK

sub infinite_loop() {

    $/ = "ENDOFMESSAGE\n";

	my $message;

    while ( $message = <main::CMD_SOCK> ) {

        # Strip message terminator _and_ untaint the data
        # I trust the source! and the setuid/setguid is downgrading!
        if ($message =~ m/(.*)(ENDOFMESSAGE)/s ) {
            $message = $1;
        }

        # Got the message, do something about it. 

            # Now find which command to execute

            SWITCH: {

                # --- Commands that do not require a specific user

                # Are you still there? from NJS. No execution of commands
                # just reply with blank to NJS, so no need to setuid
                $message =~ /#TSI_PING\n/  && do {
                                                last SWITCH;
                                              };

                # --- Now switch user for those that should be run as a user

                # Never execute any command as root, always setuid to a user
                # and if this fails abort execution of this command

                # Find which user we should be setting UID to
                # look for the line: #TSI_IDENTITY user_name project name
				my $user;
				my $project;

				if($message =~ m/#TSI_IDENTITY (\S+) (\S+)/) {
					$user = $1;
					$project = $2;
				}
				else {
					failed_report("No user identity found in: $message ---");
					last SWITCH;
				}

                last SWITCH unless become_user($user,$project) == 1;

				dump2file($message);

                $message =~ /#TSI_GETSTATUSLISTING\n/  && do {
                                                get_status_listing($message);
                                                last SWITCH;
                                              };

                $message =~ /#TSI_SUBMIT\n/  && do {
                                                submit($message);
                                                last SWITCH;
                                              };

                $message =~ /#TSI_EXECUTESCRIPT\n/  && do {
                                                execute_script($message);
                                                last SWITCH;
                                              };

                $message =~ /#TSI_ENDPROCESSING\n/  && do {
                                                end_processing($message);
                                                last SWITCH;
                                              };

                $message =~ /#TSI_PUTFILES\n/  && do {
                                                put_files($message);
                                                last SWITCH;
                                              };
                $message =~ /#TSI_GETDIRECTORY\n/  && do {
                                                get_directory($message);
                                                last SWITCH;
                                              };

                $message =~ /#TSI_ABORTJOB\n/  && do {
                                                abort_job($message);
                                                last SWITCH;
                                              };

                $message =~ /#TSI_CANCELJOB\n/  && do {
                                                cancel_job($message);
                                                last SWITCH;
                                              };

                $message =~ /#TSI_HOLDJOB\n/  && do {
                                                hold_job($message);
                                                last SWITCH;
                                              };

                $message =~ /#TSI_RESUMEJOB\n/  && do {
                                                resume_job($message);
                                                last SWITCH;
                                              };

                failed_report("Unrecognised NJS command: $message ---");

            }

            restore_id();


        $message  = "";

        print main::CMD_SOCK "ENDOFMESSAGE\n";

        $/ = "ENDOFMESSAGE\n";
    }

report_and_die("End of inputs from NJS, finishing execution");

exit;

}
#
# $Id: MainLoop.pm,v 1.17 2002/04/09 17:42:09 svdb Exp $
#
# 04-05-01 Sven van den Berghe fecit Created from 3.0 TSI for Unicore 3.5
#
#                  Copyright (c) Fujitsu Ltd 2000 - 2002
#
#     Use and distribution is subject to the Community Source Licence.
#                                                       
# Further information is available from www.unicore.org or arcon@fecit.co.uk.
#
