package GetStatusListing;

###############################################################################
# UXP/V
#
#    developed on a VPP at fecit (tested) Sven van den Berghe
#    adapted for Hitachi sr8000 at LRZ Muenchen, Irene Geiseler (ige) 
#
###############################################################################

require Exporter;
@ISA = qw(Exporter);

@EXPORT_OK = qw(get_status_listing);

use Reporting qw(debug_report failed_report ok_report start_report command_report report_and_die);

use strict;

# Queries the BSS for the state of (at least) all Unicore jobs
# and returns a list of their identifier together with the
# stats.
#
# Called in response to a #TSI_JOBQUERY in the script

# uses functions from Reporting

# No args
#
# Returns void to TSI
#         fail/success to NJS and a list of jobs
#
#        First line is "QSTAT",
#        followed by a line per found job, first word is BSS job identifier
#        and the second word is one of RUNNING, QUEUED or SUSPENDED
#
#        List must include all Unicore jobs (but can contain  extra jobs)
#
#
sub get_status_listing {
### new variables, ige ###
  my($command, $output, $result, $bssid, @lines, $lines, $state, $ustate) = ("", "", "", "", "", "", "", "");

  start_report("Finding all jobs on NQS");

  # Form the request command. 
  $command = "$main::qstat_cmd";

    command_report($command);

    $output = `($command) 2>&1`;

    # Parse output
    if($? != 0) {
        failed_report($output);
    }
    else {
        # Command succeeded. Parse the output and return
        # a line for each job found with two words, the first
        # is the BSS id the second its _Unicore_ state
#
### begin of modification, ige ####
        # The output format of the lines we want is:
# REQUEST         OWNER       QUEUE     NODE PARTNAME      E-TIME   MEM     STATE
# 41858.sr8000.lr ownername    N1        1    MEMORY        65100   14GB    RUNNING
# 41859.sr8000.lr ownername    N1        1    MEMORY        65100   14GB    RUNNING 

        $result = "QSTAT\n";

        @lines = split /\n/, $output;
            foreach $lines (@lines) 
            {
                 if ($lines =~ /^(\d+)\.\S+\s+\S+\s+\S+\s+\d+\s+\w+\s+\d+\s+\S+\s+(\w+)/)
                 {
                 $bssid = $1;
                 $state = $2;
            # Catches all known values, treats pipe queue states as queued as
            # they are intermediate and we do not know what will happen to the
            # on the destination queues.

               $ustate = "UNKNOWN";
               $ustate = "QUEUED"  if $state =~ /QUEUED|WAITING|ARRIVING|ROUTING|DEPARTING|INTRANSIT|STAGING|FREEZING|FROZEN|FRESTART/;
               $ustate = "SUSPENDED"  if $state =~ /HOLDING/;
               $ustate = "RUNNING" if $state =~ /RUNNING|EXITING/ ;

            # Add to the returned lines
                 $result = "$result $bssid $ustate\n";
                 }
            }
### end of modification, ige ###
#
        print main::CMD_SOCK $result;
        debug_report("qstat executed OK");
    }

}
#
# $Id: GetStatusListing.pm,v 1.11 2001/08/28 16:56:01 svdb Exp $
#
# 04-05-01 Sven van den Berghe fecit Created from 3.0 TSI for Unicore 3.5
#
#                       Copyright 1999 to 2001 Fujitsu European Centre for Information Technology Ltd.
#  This is unpublished proprietary source code which is the property of Fujitsu European Centre for Information Technology Ltd.
#
